@@ -31,60 +31,123 @@
 #include "ms-excel.h"
 #include "ms-excel-biff.h"
 
-#define STRNPRINTF(ptr,n){ int xxxlp; printf ("'") ; for (xxxlp=0;xxxlp<(n);xxxlp++) printf ("%c", (ptr)[xxxlp]) ; printf ("'\n") ; }
-
 /* Forward references */
 static MS_EXCEL_SHEET *ms_excel_sheet_new (MS_EXCEL_WORKBOOK * wb, char *name) ;
 static void ms_excel_workbook_attach (MS_EXCEL_WORKBOOK * wb, MS_EXCEL_SHEET * ans) ;
 
-/*
- * FIXME: This needs proper unicode support ! current support is a guess 
- * see S59D47.HTM for full description
- */
-static char *
+/**
+ * Generic 16 bit int index pointer functions.
+ **/
+static guint
+biff_guint16_hash (const guint16 *d)
+{ return *d ; }
+static guint
+biff_guint32_hash (const guint32 *d)
+{ return *d ; }
+
+static gint
+biff_guint16_equal (const guint16 *a, const guint16 *b)
+{
+	if (*a==*b) return 1 ;
+	return 0 ;
+}
+static gint
+biff_guint32_equal (const guint32 *a, const guint32 *b)
+{
+	if (*a==*b) return 1 ;
+	return 0 ;
+}
+
+/**
+ *  This function takes a length argument as Biff V7 has a byte length
+ * ( seemingly ).
+ *  FIXME: see S59D47.HTM for full description
+ **/
+char *
 biff_get_text (BYTE * ptr, int length)
 {
-	int lp, unicode;
+	int lp ;
 	char *ans;
-	BYTE *inb;
+	BYTE header ;
+	gboolean high_byte ;
+	gboolean ext_str ;
+	gboolean rich_str ;
 
 	if (!length)
 		return 0;
 
 	ans = (char *) g_malloc (sizeof (char) * length + 2);
 
-	/*
-	 * Magic unicode number 
-	 */
-	unicode = (ptr[0] == 0x1);
-	inb = unicode ? ptr + 1 : ptr;
-
-	for (lp = 0; lp < length; lp++){
-		ans [lp] = (char) *inb;
-		inb += unicode ? 2 : 1;
+	header = BIFF_GETBYTE(ptr) ;
+	/* I assume that this header is backwards compatible with raw ASCII */
+	if (((header & 0xf0) == 0) &&
+	    ((header & 0x02) == 0)) /* Its a proper Unicode header grbit byte */
+	{
+		high_byte = (header & 0x1) != 0 ;
+		ext_str   = (header & 0x4) != 0 ;
+		rich_str  = (header & 0x8) != 0 ;
+		ptr++ ;
 	}
-	ans [lp] = 0;
+	else /* Some assumptions: FIXME ? */
+	{
+		high_byte = 0 ;
+		ext_str   = 0 ;
+		rich_str  = 0 ;
+	}
+
+	/* A few friendly warnings */
+	if (high_byte)
+		printf ("FIXME: unicode support unimplemented: truncating\n") ;
+	if (rich_str) /* The data for this appears after the string */
+	{
+		guint16 formatting_runs = BIFF_GETWORD(ptr) ;
+		ptr+= 2 ;
+		printf ("FIXME: rich string support unimplemented: discarding %d runs\n", formatting_runs) ;
+	}
+	if (ext_str) /* NB this data always comes after the rich_str data */
+	{
+		guint32 len_ext_rst = BIFF_GETLONG(ptr) ;
+		printf ("FIXME: extended string support unimplemented: ignoring %d bytes\n", len_ext_rst) ;
+		ptr+= 4 ;
+	}
+
+	for (lp = 0; lp < length; lp++)
+	{
+		ans [lp] = (char) *ptr ;
+		ptr += high_byte ? 2 : 1;
+	}
+	ans[lp] = 0;
 	return ans;
 }
 
 static char *
 biff_get_global_string(MS_EXCEL_SHEET *sheet, int number)
 {
-	char *temp;
-	int length, k;
-	
 	MS_EXCEL_WORKBOOK *wb = sheet->wb;
 	
         if (number >= wb->global_string_max)
 		return "Too Weird";
 	
-	temp= wb->global_strings;
-        for (k = 0; k < number; k++){
-		length= BIFF_GETWORD (temp);
-		temp+= length + 3;
-        }
-        length = BIFF_GETWORD(temp);
-        return biff_get_text (temp+3, length);
+	return wb->global_strings[number] ;
+}
+
+char *
+biff_get_error_text (guint8 err)
+{
+	char *buf ;
+	switch (err)
+	{
+	case 0:  buf = "#NULL!" ;  break ;
+	case 7:  buf = "#DIV/0!" ; break ;
+	case 15: buf = "#VALUE!" ; break ;
+	case 23: buf = "#REF!" ;   break ;
+	case 29: buf = "#NAME?" ;  break ;
+	case 36: buf = "#NUM!" ;   break ;
+	case 42: buf = "#N/A" ;    break ;
+	default:
+		buf = "#UNKNOWN!" ;
+	}
+	return buf ;
 }
 
 /**
@@ -181,6 +244,9 @@ ms_biff_bof_data_destroy (BIFF_BOF_DATA * data)
 	g_free (data);
 }
 
+/**
+ * See S59D61.HTM
+ **/
 static void
 biff_boundsheet_data_new (MS_EXCEL_WORKBOOK *wb, BIFF_QUERY * q, eBiff_version ver)
 {
@@ -229,9 +295,8 @@ biff_boundsheet_data_new (MS_EXCEL_WORKBOOK *wb, BIFF_QUERY * q, eBiff_version v
 		ans->hidden = eBiffHVisible;
 		break;
 	}
-	if (ver == eBiffV8){
+	if (ver == eBiffV8) {
 		int strlen = BIFF_GETWORD (q->data + 6);
-
 		ans->name = biff_get_text (q->data + 8, strlen);
 	} else {
 		int strlen = BIFF_GETBYTE (q->data + 6);
@@ -261,25 +326,6 @@ biff_boundsheet_data_destroy (gpointer key, BIFF_BOUNDSHEET_DATA *d, gpointer us
 	return 1 ;
 }
 
-static guint
-biff_boundsheet_data_stream_hash (const guint32 *d)
-{
-        return *d ;
-}
-
-static guint
-biff_boundsheet_data_index_hash (const guint16 *d)
-{
-        return *d ;
-}
-
-static gint
-biff_boundsheet_data_compare (const BIFF_BOUNDSHEET_DATA *a, const BIFF_BOUNDSHEET_DATA *b)
-{
-	if (a->index==b->index) return 1 ;
-	return 0 ;
-}
-
 /**
  * NB. 'fount' is the correct, and original _English_
  **/
@@ -349,19 +395,6 @@ biff_font_data_destroy (gpointer key, BIFF_FONT_DATA *d, gpointer userdata)
 	return 1 ;
 }
 
-static guint
-biff_font_data_hash (const guint16 *d)
-{
-        return *d ;
-}
-
-static gint
-biff_font_data_compare (const BIFF_FONT_DATA *a, const BIFF_FONT_DATA *b)
-{
-	if (a->index==b->index) return 1 ;
-	return 0 ;
-}
-
 static MS_EXCEL_PALETTE *
 ms_excel_palette_new (BIFF_QUERY * q)
 {
@@ -822,25 +855,13 @@ biff_xf_data_destroy (gpointer key, BIFF_XF_DATA *d, gpointer userdata)
 	return 1 ;
 }
 
-static guint
-biff_xf_data_hash (const guint16 *d)
-{
-        return *d ;
-}
-
-static gint
-biff_xf_data_compare (const BIFF_XF_DATA *a, const BIFF_XF_DATA *b)
-{
-	if (a->index==b->index) return 1 ;
-	return 0 ;
-}
-
 static MS_EXCEL_SHEET *
 ms_excel_sheet_new (MS_EXCEL_WORKBOOK * wb, char *name)
 {
 	MS_EXCEL_SHEET *ans = (MS_EXCEL_SHEET *) g_malloc (sizeof (MS_EXCEL_SHEET));
 
 	ans->gnum_sheet = sheet_new (wb->gnum_wb, name);
+	ans->blank = 1 ;
 	ans->wb = wb;
 	ans->array_formulae = 0;
 	return ans;
@@ -858,6 +879,7 @@ ms_excel_sheet_insert (MS_EXCEL_SHEET * sheet, int xfidx, int col, int row, char
 	Cell *cell = sheet_cell_fetch (sheet->gnum_sheet, col, row);
 
 	/* NB. cell_set_text _certainly_ strdups *text */
+	sheet->blank = 0 ;
 	cell_set_text (cell, text);
 	ms_excel_set_cell_xf (sheet, cell, xfidx);
 }
@@ -892,17 +914,17 @@ ms_excel_workbook_new ()
 	ans->extern_sheets = NULL ;
 	ans->gnum_wb = NULL;
 	/* Boundsheet data hashed twice */
-	ans->boundsheet_data_by_stream = g_hash_table_new ((GHashFunc)biff_boundsheet_data_stream_hash,
-							   (GCompareFunc)biff_boundsheet_data_compare) ;
-	ans->boundsheet_data_by_index = g_hash_table_new ((GHashFunc)biff_boundsheet_data_index_hash,
-							  (GCompareFunc)biff_boundsheet_data_compare) ;
-	ans->font_data = g_hash_table_new ((GHashFunc)biff_font_data_hash,
-					   (GCompareFunc)biff_font_data_compare) ;
+	ans->boundsheet_data_by_stream = g_hash_table_new ((GHashFunc)biff_guint32_hash,
+							   (GCompareFunc)biff_guint32_equal) ;
+	ans->boundsheet_data_by_index = g_hash_table_new ((GHashFunc)biff_guint16_hash,
+							  (GCompareFunc)biff_guint16_equal) ;
+	ans->font_data = g_hash_table_new ((GHashFunc)biff_guint16_hash,
+					   (GCompareFunc)biff_guint16_equal) ;
 	ans->excel_sheets = NULL;
-	ans->XF_style_records = g_hash_table_new ((GHashFunc)biff_xf_data_hash,
-						  (GCompareFunc)biff_xf_data_compare) ;
-	ans->XF_cell_records = g_hash_table_new ((GHashFunc)biff_xf_data_hash,
-						 (GCompareFunc)biff_xf_data_compare) ;
+	ans->XF_style_records = g_hash_table_new ((GHashFunc)biff_guint16_hash,
+						  (GCompareFunc)biff_guint16_equal) ;
+	ans->XF_cell_records = g_hash_table_new ((GHashFunc)biff_guint16_hash,
+						 (GCompareFunc)biff_guint16_equal) ;
 	ans->palette = NULL;
 	ans->global_strings = NULL;
 	ans->global_string_max = 0;
@@ -1051,23 +1073,34 @@ ms_excel_read_cell (BIFF_QUERY * q, MS_EXCEL_SHEET * sheet)
 			}
 		}
 		break;
- 	case BIFF_HEADER: /* FIXME : S59D94 Microsoft failed to document this correctly  */
- 		printf ("Header\n");
+ 	case BIFF_HEADER: /* FIXME : S59D94 */
+	{
+		char *str ;
 		if (q->length)
-			dump (q->data+1, BIFF_GETBYTE(q->data+0));
+		{
+			printf ("Header '%s'\n", (str=biff_get_text (q->data+1,
+								     BIFF_GETBYTE(q->data)))) ;
+			g_free(str) ;
+		}
  		break;
- 	case BIFF_FOOTER: /* FIXME : S59D8D Microsoft failed to document this correctly  */
- 		printf ("Footer\n");
+	}
+ 	case BIFF_FOOTER: /* FIXME : S59D8D */
+	{
+		char *str ;
 		if (q->length)
-			dump (q->data+1, BIFF_GETBYTE(q->data+0));
+		{
+			printf ("Footer '%s'\n", (str=biff_get_text (q->data+1,
+								     BIFF_GETBYTE(q->data)))) ;
+			g_free(str) ;
+		}
  		break;
+	}
 	case BIFF_RSTRING:
 	{
 		char *txt ;
 		/*
 		  printf ("Cell [%d, %d] = ", EX_GETCOL(q), EX_GETROW(q));
 		  dump (q->data, q->length);
-		  STRNPRINTF(q->data + 8, EX_GETSTRLEN(q)); 
 		*/
 		printf ("Rstring\n") ;
 		ms_excel_sheet_insert (sheet, EX_GETXF (q), EX_GETCOL (q), EX_GETROW (q),
@@ -1082,6 +1115,7 @@ ms_excel_read_cell (BIFF_QUERY * q, MS_EXCEL_SHEET * sheet)
 									 * FIXME GETDOUBLE is not endian independant 
 									 */
 			dump (q->data, q->length);
+/*			snprintf (buf, 64, "NUM %f", num); */
 			snprintf (buf, 64, "%f", num);
 			ms_excel_sheet_insert (sheet, EX_GETXF (q), EX_GETCOL (q), EX_GETROW (q), buf);
 			break;
@@ -1099,18 +1133,17 @@ ms_excel_read_cell (BIFF_QUERY * q, MS_EXCEL_SHEET * sheet)
 			} type;
 
 			number = BIFF_GETLONG (q->data + 6);
-			printf ("RK number : 0x%x, length 0x%x\n", q->opcode, q->length);
 			type = (number & 0x3);
+			printf ("RK number : 0x%x, length 0x%x\n", q->opcode, q->length);
 			printf ("position [%d,%d] = %x ( type %d )\n", EX_GETCOL (q), EX_GETROW (q), number, type);
+			dump (q->data, q->length);
 			switch (type){
 			case eIEEE:
-				dump (q->data, q->length);
 				tmp[0] = 0;
 				tmp[1] = number & 0xfffffffc;
 				answer = BIFF_GETDOUBLE (((BYTE *) tmp));
 				break;
 			case eIEEEx10:
-				dump (q->data, q->length);
 				tmp[0] = 0;
 				tmp[1] = number & 0xfffffffc;
 				answer = BIFF_GETDOUBLE (((BYTE *) tmp));
@@ -1126,6 +1159,7 @@ ms_excel_read_cell (BIFF_QUERY * q, MS_EXCEL_SHEET * sheet)
 				printf ("You don't exist go away\n");
 				answer = 0;
 			}
+/*			sprintf (buf, "RK %d %f", type, answer);*/
 			sprintf (buf, "%f", answer);
 			ms_excel_sheet_insert (sheet, EX_GETXF (q), EX_GETCOL (q), EX_GETROW (q), buf);
 		}
@@ -1145,12 +1179,13 @@ ms_excel_read_cell (BIFF_QUERY * q, MS_EXCEL_SHEET * sheet)
 		 * printf ("Row %d formatting\n", EX_GETROW(q)); 
 		 */
 		break;
-	case BIFF_FORMULA:	/*
-				 * FIXME: S59D8F.HTM 
-				 */
 	case BIFF_ARRAY:	/*
 				 * FIXME: S59D57.HTM 
 				 */
+		printf ("Array Formula\n") ;
+	case BIFF_FORMULA:	/*
+				 * FIXME: S59D8F.HTM 
+				 */
 		/*
 		 * case BIFF_NAME FIXME: S59DA9.HTM 
 		 */
@@ -1159,16 +1194,73 @@ ms_excel_read_cell (BIFF_QUERY * q, MS_EXCEL_SHEET * sheet)
 	case BIFF_LABELSST:
 	{
 		char *str;
+		guint32 idx = BIFF_GETLONG (q->data + 6) ;
 		
-		str = biff_get_global_string (sheet, BIFF_GETLONG (q->data + 6));
+		if (idx >= sheet->wb->global_string_max)
+		{
+			printf ("string index 0x%x out of range\n", idx) ;
+			break ;
+		}
 		
+		str = sheet->wb->global_strings[idx] ;
 		ms_excel_sheet_insert (sheet, EX_GETXF (q), EX_GETCOL (q), EX_GETROW (q), str);
-		g_free (str);
                 break;
 	}
 	default:
 		switch (q->opcode)
 		{
+		case BIFF_NAME: /* FIXME: S59DA9.HTM */
+		{
+			guint16 flags = BIFF_GETWORD(q->data) ;
+			guint16 fn_grp_idx ;
+			guint8  kb_shortcut = BIFF_GETBYTE(q->data+2);
+			guint8  name_len = BIFF_GETBYTE(q->data+3) ;
+			guint16 name_def_len  = BIFF_GETWORD(q->data+4) ;
+			guint8* name_def_data = q->data+14+name_def_len ;
+			guint16 sheet_idx = BIFF_GETWORD(q->data+6) ;
+			guint16 ixals = BIFF_GETWORD(q->data+8) ; /* dup */
+			guint8  menu_txt_len = BIFF_GETBYTE(q->data+10) ;
+			guint8  descr_txt_len = BIFF_GETBYTE(q->data+11) ;
+			guint8  help_txt_len = BIFF_GETBYTE(q->data+12) ;
+			guint8  status_txt_len = BIFF_GETBYTE(q->data+13) ;
+			char *name, *menu_txt, *descr_txt, *help_txt, *status_txt ;
+			guint8 *ptr ;
+
+			g_assert (ixals==sheet_idx) ;
+			ptr = q->data + 14 ;
+			name = biff_get_text (ptr, name_len) ;
+			ptr+= name_len + name_def_len ;
+			menu_txt = biff_get_text (ptr, menu_txt_len) ;
+			ptr+= menu_txt_len ;
+			descr_txt = biff_get_text (ptr, descr_txt_len) ;
+			ptr+= descr_txt_len ;
+			help_txt = biff_get_text (ptr, help_txt_len) ;
+			ptr+= help_txt_len ;
+			status_txt = biff_get_text (ptr, status_txt_len) ;
+
+			printf ("Name record : '%s', '%s', '%s', '%s', '%s'\n", name, menu_txt, descr_txt,
+				help_txt, status_txt) ;
+			dump (name_def_data, name_def_len) ;
+
+			/* Unpack flags */
+			fn_grp_idx = (flags&0xfc0)>>6 ;
+			if ((flags&0x0001) != 0)
+				printf (" Hidden") ;
+			if ((flags&0x0002) != 0)
+				printf (" Function") ;
+			if ((flags&0x0004) != 0)
+				printf (" VB-Proc") ;
+			if ((flags&0x0008) != 0)
+				printf (" Proc") ;
+			if ((flags&0x0010) != 0)
+				printf (" CalcExp") ;
+			if ((flags&0x0020) != 0)
+				printf (" BuiltIn") ;
+			if ((flags&0x1000) != 0)
+				printf (" BinData") ;
+			printf ("\n") ;
+			break ;
+		}
 		case BIFF_STRING: /* FIXME: S59DE9.HTM */
 		{
 			char *txt ;
@@ -1179,22 +1271,9 @@ ms_excel_read_cell (BIFF_QUERY * q, MS_EXCEL_SHEET * sheet)
 		case BIFF_BOOLERR: /* S59D5F.HTM */
 		{
 			if (BIFF_GETBYTE(q->data + 7)) /* Error */
-			{
-				char *buf ;
-				switch (BIFF_GETBYTE(q->data + 6))
-				{
-					case 0:  buf = "#NULL!" ;  break ;
-					case 7:  buf = "#DIV/0!" ; break ;
-					case 15: buf = "#VALUE!" ; break ;
-					case 23: buf = "#REF!" ;   break ;
-					case 29: buf = "#NAME?" ;  break ;
-					case 36: buf = "#NUM!" ;   break ;
-					case 42: buf = "#N/A" ;    break ;
-				default:
-					buf = "#UNKNOWN!" ;
-				}
-				ms_excel_sheet_insert (sheet, EX_GETXF (q), EX_GETCOL (q), EX_GETROW (q), buf) ;
-			}
+				ms_excel_sheet_insert (sheet, EX_GETXF (q), EX_GETCOL (q)
+						       , EX_GETROW (q), 
+						       biff_get_error_text (BIFF_GETBYTE(q->data + 6))) ;
 			else /* Boolean */
 			{
 				if (BIFF_GETBYTE(q->data + 6))
@@ -1221,7 +1300,7 @@ ms_excel_read_sheet (MS_EXCEL_SHEET *sheet, BIFF_QUERY * q, MS_EXCEL_WORKBOOK *
 	while (ms_biff_query_next (q)){
 		switch (q->ls_op){
 		case BIFF_EOF:
-			if (q->streamPos == blankSheetPos)
+			if (q->streamPos == blankSheetPos || sheet->blank)
 			{
 				printf ("Blank sheet\n");
 				ms_excel_workbook_detach (sheet->wb, sheet) ;
@@ -1388,8 +1467,8 @@ ms_excelReadWorkbook (MS_OLE * file)
 				{
 					BIFF_FONT_DATA *ptr;
 
-					printf ("Read Font\n");
-					dump (q->data, q->length);
+/*					printf ("Read Font\n");
+					dump (q->data, q->length); */
 					biff_font_data_new (wb, q);
 				}
 				break;
@@ -1406,13 +1485,21 @@ ms_excelReadWorkbook (MS_OLE * file)
 				biff_xf_data_new (wb, q, ver->version) ;
 				break;
 			case BIFF_SST: /* see S59DE7.HTM */
-				wb->global_strings = g_malloc(q->length-8);
-				memcpy(wb->global_strings, q->data+8, q->length-8);
+			{
+				int length, k ;
+				char *temp ;
 				wb->global_string_max = BIFF_GETLONG(q->data+4);
-				printf("There are apparently %d strings\n",
-				       wb->global_string_max);
-				dump (q->data+8, q->length-8) ;
+				wb->global_strings = g_new (char *, wb->global_string_max) ;
+
+				temp = q->data + 8 ;
+				for (k = 0; k < wb->global_string_max; k++)
+				{
+					length = BIFF_GETWORD (temp) ;
+					wb->global_strings[k] = biff_get_text (temp+2, length) ;
+					temp+= length + 3 ;
+				}
 				break;
+			}
 			case BIFF_EXTERNSHEET:
 			{
 				if ( ver->version == eBiffV8 )
