@@ -25,6 +25,7 @@ typedef struct _BIFF_BOUNDSHEET_DATA
 typedef struct _MS_EXCEL_SHEET
 {
 	int index ;
+	int blank ;
 	Sheet *gnum_sheet ;
 	struct _MS_EXCEL_WORKBOOK *wb ;
 	eBiff_version ver ;
@@ -72,11 +73,7 @@ typedef struct _MS_EXCEL_WORKBOOK
 	BIFF_EXTERNSHEET_DATA *extern_sheets ;
 	guint16 num_extern_sheets ;
 	MS_EXCEL_PALETTE *palette ;
-	/**
-	 *    Global strings kludge, works for me,
-	 * Caveat Emptor -- njl195@zepler.org
-	 **/
-	char *global_strings;
+	char **global_strings;
 	int global_string_max;
 
 	/**
@@ -86,5 +83,6 @@ typedef struct _MS_EXCEL_WORKBOOK
 } MS_EXCEL_WORKBOOK ;
 
 extern char* biff_get_externsheet_name (MS_EXCEL_WORKBOOK *wb, guint16 idx, gboolean get_first) ;
-
+extern char* biff_get_text (BYTE *ptr, int length) ;
+extern char* biff_get_error_text (guint8 err) ;
 #endif
