@@ -591,7 +591,7 @@ make_color_picker_notify (GtkWidget *widget, GtkWidget *prop_win)
 static GtkWidget *
 create_foreground_radio (GtkWidget *prop_win)
 {
-	GtkWidget *frame, *table, *r1, *r2;
+	GtkWidget *frame, *table, *r1, *r2, *r3;
 	int e = GTK_FILL | GTK_EXPAND;
 	
 	frame = gtk_frame_new (_("Text color"));
@@ -601,8 +601,10 @@ create_foreground_radio (GtkWidget *prop_win)
 	r1 = gtk_radio_button_new_with_label (NULL, _("None"));
 	r2 = gtk_radio_button_new_with_label_from_widget (
 		GTK_RADIO_BUTTON (r1), _("Use this color"));
+	r3 = gtk_radio_button_new_with_label_from_widget (
+		GTK_RADIO_BUTTON (r1), _("No change"));
 
-	foreground_radio_list = GTK_RADIO_BUTTON (r2)->group;
+	foreground_radio_list = GTK_RADIO_BUTTON (r3)->group;
 
 	foreground_cs = gnome_color_picker_new ();
 
@@ -610,6 +612,7 @@ create_foreground_radio (GtkWidget *prop_win)
 	
 	gtk_table_attach (GTK_TABLE (table), r1, 0, 1, 0, 1, e, 0, 4, 2);
 	gtk_table_attach (GTK_TABLE (table), r2, 0, 1, 1, 2, e, 0, 4, 2);
+	gtk_table_attach (GTK_TABLE (table), r3, 0, 1, 2, 3, e, 0, 4, 2);
 	gtk_table_attach (GTK_TABLE (table), foreground_cs,
 			  1, 2, 1, 2, 0, 0, 0, 0); 
 
@@ -646,10 +649,10 @@ create_background_radio (GtkWidget *prop_win)
 	
 	gtk_table_attach (GTK_TABLE (table), r1, 0, 1, 0, 1, e, 0, 4, 2);
 	gtk_table_attach (GTK_TABLE (table), r2, 0, 1, 1, 2, e, 0, 4, 2);
-	gtk_table_attach (GTK_TABLE (table), r3, 0, 1, 2, 3, e, 0, 4, 2);
+	/* gtk_table_attach (GTK_TABLE (table), r3, 0, 1, 2, 3, e, 0, 4, 2); */
 
 	gtk_table_attach (GTK_TABLE (table), background_cs, 1, 2, 1, 2, 0, 0, 4, 2);
-	gtk_table_attach (GTK_TABLE (table), p, 0, 2, 3, 4, GTK_FILL | GTK_EXPAND, GTK_FILL|GTK_EXPAND, 0, 0);
+	/* gtk_table_attach (GTK_TABLE (table), p, 0, 2, 3, 4, GTK_FILL | GTK_EXPAND, GTK_FILL|GTK_EXPAND, 0, 0); */
 	return frame;
 }
 
@@ -748,6 +751,8 @@ create_coloring_page (GtkWidget *prop_win, CellList *cells)
 				gtk_radio_button_select (foreground_radio_list, 1);
 				gnome_color_picker_set_d (GNOME_COLOR_PICKER (foreground_cs), rd, gd, bd, ad);
 			}
+		} else {
+			gtk_radio_button_select (foreground_radio_list, 2);
 		}
 		if (ok_back != 0){
 			if (background_flag == 0){
@@ -768,7 +773,7 @@ create_coloring_page (GtkWidget *prop_win, CellList *cells)
 	make_radio_notify_change (background_radio_list, prop_win);
 	
 	gtk_table_attach (t, fore, 0, 1, 0, 1, e, 0, 4, 4);
-/*	gtk_table_attach (t, back, 0, 1, 1, 2, e, 0, 4, 4); */
+	gtk_table_attach (t, back, 0, 1, 1, 2, e, 0, 4, 4);
 
 	gtk_widget_show_all (GTK_WIDGET (t));
 
@@ -779,6 +784,7 @@ static void
 apply_coloring_format (Style *style, Sheet *sheet, CellList *cells)
 {
 	double rd, gd, bd, ad;
+	gushort fore_change, back_change;
 	gushort fore_red, fore_green, fore_blue;
 	gushort back_red, back_green, back_blue;
 
@@ -788,25 +794,41 @@ apply_coloring_format (Style *style, Sheet *sheet, CellList *cells)
 	/*
 	 * Let's check the foreground first
 	 */
-	if (gtk_radio_group_get_selected (foreground_radio_list) == 1){
-		gnome_color_picker_get_d (GNOME_COLOR_PICKER (foreground_cs), &rd, &gd, &bd, &ad);
-
-		fore_red   = rd * 65535;
-		fore_green = gd * 65535;
-		fore_blue  = bd * 65535;
-		style->valid_flags |= STYLE_FORE_COLOR;
-	} else {
+	switch (gtk_radio_group_get_selected (foreground_radio_list)) {
+	/*
+	 * case 0 means no foreground
+	 */
+	case 0:
 		fore_red   = 0;
 		fore_green = 0;
 		fore_blue  = 0;
 		style->valid_flags &= ~STYLE_FORE_COLOR;
+		fore_change = TRUE;
+		break;
+	/*
+	 * case 1 means colored foreground
+	 */
+	case 1:
+		gnome_color_picker_get_d (GNOME_COLOR_PICKER (foreground_cs), &rd, &gd, &bd, &ad);
+		fore_red   = rd * 65535;
+		fore_green = gd * 65535;
+		fore_blue  = bd * 65535;
+		style->valid_flags |= STYLE_FORE_COLOR;
+		fore_change = TRUE;
+		break;
+	/*
+	 * case 2 means no change
+	 */
+	case 2:
+		fore_change = FALSE;
+		break;
 	}
 
 	/*
 	 * Now, the background
 	 * FIXME: What about the cell pattern?
 	 */
-	switch (gtk_radio_group_get_selected (background_radio_list)){
+	switch (gtk_radio_group_get_selected (background_radio_list)) {
 	/*
 	 * case 0 means no background
 	 */
@@ -834,7 +856,7 @@ apply_coloring_format (Style *style, Sheet *sheet, CellList *cells)
 	/*
 	 * case 2 means a pattern background
 	 */
-	default:
+	case 2:
 		back_red = 0xffff;
 		back_green = 0xffff;
 		back_blue = 0xffff;
@@ -848,12 +870,16 @@ apply_coloring_format (Style *style, Sheet *sheet, CellList *cells)
 	for (; cells; cells = cells->next){
 		cell = cells->data;
 
-		cell_set_foreground (cell, fore_red, fore_green, fore_blue);
+		if (fore_change==TRUE) {
+			cell_set_foreground (cell, fore_red, fore_green, fore_blue);
+		}
 		cell_set_background (cell, back_red, back_green, back_blue);
 /*		cell_set_pattern    (cell, 2); */
 	}
 
-	style->fore_color  = style_color_new (fore_red, fore_green, fore_blue);
+	if (fore_change==TRUE) {
+		style->fore_color  = style_color_new (fore_red, fore_green, fore_blue);
+	}
 	style->back_color  = style_color_new (back_red, back_green, back_blue);
 }
 
