@@ -245,12 +245,21 @@ move_vertical_selection (GnumericSheet *gsheet, int count)
 int
 gnumeric_sheet_can_move_cursor (GnumericSheet *gsheet)
 {
-	GtkEntry *entry = GTK_ENTRY (gsheet->entry);
-	int cursor_pos = GTK_EDITABLE (entry)->current_pos;
+	GtkEntry *entry;
+	int cursor_pos;
 
-	if (gsheet->selecting_cell)
+	g_return_val_if_fail (gsheet != NULL, FALSE);
+	g_return_val_if_fail (GNUMERIC_IS_SHEET (gsheet), FALSE);
+
+	if (!gsheet->sheet_view->sheet->editing)
+		return FALSE;
+	
+	if (gsheet->item_editor && gsheet->selecting_cell)
 		return TRUE;
 	
+	entry = GTK_ENTRY (gsheet->entry);
+	cursor_pos = GTK_EDITABLE (entry)->current_pos;
+	
 	if (entry->text [0] != '=')
 		return FALSE;
 	if (cursor_pos == 0)
@@ -757,6 +766,7 @@ gnumeric_sheet_key_mode_sheet (GnumericSheet *gsheet, GdkEventKey *event)
 		
 	case GDK_F2:
 		gtk_window_set_focus (GTK_WINDOW (wb->toplevel), wb->ea_input);
+		sheet->editing = TRUE;
 		/* fall down */
 
 	default:
@@ -771,10 +781,10 @@ gnumeric_sheet_key_mode_sheet (GnumericSheet *gsheet, GdkEventKey *event)
 		gnumeric_sheet_stop_cell_selection (gsheet);
 		
 		/* Forward the keystroke to the input line */
-		gtk_widget_event (gsheet->entry, (GdkEvent *) event);
-		
+		return gtk_widget_event (gsheet->entry, (GdkEvent *) event);
 	}
-	return 1;
+
+	return TRUE;
 }
 
 static gint
@@ -813,7 +823,7 @@ gnumeric_sheet_key (GtkWidget *widget, GdkEventKey *event)
 		return gnumeric_sheet_key_mode_object (gsheet, event);
 
 	default:
-		return TRUE;
+		return FALSE;
 	}
 }
 
