@@ -1,10 +1,16 @@
-#ifndef ITEM_GRID_H
-#define ITEM_GRID_H
+#ifndef GNUMERIC_ITEM_GRID_H
+#define GNUMERIC_ITEM_GRID_H
 
 #define ITEM_GRID(obj)          (GTK_CHECK_CAST((obj), item_grid_get_type (), ItemGrid))
 #define ITEM_GRID_CLASS(k)      (GTK_CHECK_CLASS_CAST ((k), item_grid_get_type ()))
 #define IS_ITEM_GRID(o)         (GTK_CHECK_TYPE((o), item_grid_get_type ()))
 
+typedef enum {
+	ITEM_GRID_NO_SELECTION,
+	ITEM_GRID_SELECTING_CELL_RANGE,
+	ITEM_GRID_SELECTING_FORMULA_RANGE
+} ItemGridSelectionType;
+
 typedef struct {
 	GnomeCanvasItem canvas_item;
 
@@ -15,7 +21,7 @@ typedef struct {
 	ColType    left_col;
 	RowType    top_row;
 
-	int        selecting;
+	ItemGridSelectionType selecting;
 	
 	/* Offset from spreadsheet origin in units */
 	long       top_offset;
@@ -30,6 +36,11 @@ typedef struct {
 	GdkColor   default_color;
 
 	int        visual_is_paletted;
+
+	/* Sliding scroll */
+	int        sliding;	/* a gtk_timeout tag, -1 means not set */
+	int        sliding_col, sliding_row;
+	int        sliding_x, sliding_y;
 } ItemGrid;
 
 GtkType item_grid_get_type (void);
@@ -40,4 +51,4 @@ typedef struct {
 	GnomeCanvasItemClass parent_class;
 } ItemGridClass;
 
-#endif
+#endif /* GNUMERIC_ITEM_GRID_H */
